unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry, BlockCiphers, EncryptedRegistry;

type
  TForm1 = class(TForm)
    txtPath: TEdit;
    lblKey: TLabel;
    lblPath: TLabel;
    lblName: TLabel;
    txtName: TEdit;
    cmdGetValue: TButton;
    cmdSetValue: TButton;
    cboKey: TComboBox;
    cmdClose: TButton;
    lblDataType: TLabel;
    cboDataType: TComboBox;
    cmdQueryDataType: TButton;
    procedure FormCreate(Sender: TObject);
    procedure cmdGetValueClick(Sender: TObject);
    procedure cmdSetValueClick(Sender: TObject);
    procedure cmdCloseClick(Sender: TObject);
    procedure cmdQueryDataTypeClick(Sender: TObject);
  private
    function CurrentKey: HKEY;
    function CurrentValue: string;
    procedure SetValue(const Value: string);
    procedure PositionRegistry(Reg: TEncryptedRegistry; CanCreate: Boolean);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  SInvalidItemIndex = 'Invalid ItemIndex: %d';
  EncryptionKey: Int64 = 561;

function TForm1.CurrentKey: HKEY;
begin
  case cboKey.ItemIndex of
    0: Result := HKEY_CURRENT_USER;
    1: Result := HKEY_LOCAL_MACHINE;
    2: Result := HKEY_CLASSES_ROOT;
    3: Result := HKEY_USERS;
    4: Result := HKEY_PERFORMANCE_DATA;
    5: Result := HKEY_CURRENT_CONFIG;
    6: Result := HKEY_DYN_DATA;
  else
    raise Exception.CreateFmt(SInvalidItemIndex, [cboKey.ItemIndex]);
  end;
end;

function TForm1.CurrentValue: string;
const
  BoolToStr: array [Boolean] of string = ('False', 'True');
var
  Reg: TEncryptedRegistry;
  N: string;
  L: Integer;
begin
  N := txtName.Text;
  Reg := TEncryptedRegistry.Create(TBlowfishCipher, EncryptionKey,
                                   SizeOf(EncryptionKey));
  try
    PositionRegistry(Reg, False);
    case cboDataType.ItemIndex of
      0: Result := Reg.ReadString(N);
      1: Result := FloatToStr(Reg.ReadFloat(N));
      2: Result := IntToStr(Reg.ReadInteger(N));
      3: Result := CurrToStr(Reg.ReadCurrency(N));
      4: Result := BoolToStr[Reg.ReadBool(N)];
      5: Result := DateToStr(Reg.ReadDate(N));
      6: Result := DateTimeToStr(Reg.ReadDateTime(N));
      7: Result := TimeToStr(Reg.ReadTime(N));
      8: begin
        L := Reg.GetDataSize(N);
        SetLength(Result, L);
        Reg.ReadBinaryData(N, Result[1], Length(Result));
      end;
    else
      raise Exception.CreateFmt(SInvalidItemIndex, [cboDataType.ItemIndex]);
    end;
  finally
    Reg.Free;
  end;
end;

function StrToBool(const S: string): Boolean;
begin
  if CompareText(S, 'True') = 0 then
    Result := True
  else if CompareText(S, 'False') = 0 then
    Result := False
  else begin
    raise Exception.CreateFmt('Invalid Boolean string: %s', [S]);
  end;
end;

procedure TForm1.SetValue(const Value: string);
var
  Reg: TEncryptedRegistry;
  N, S: string;
begin
  N := txtName.Text;
  Reg := TEncryptedRegistry.Create(TBlowfishCipher, EncryptionKey,
                                   SizeOf(EncryptionKey));
  try
    PositionRegistry(Reg, True);
    case cboDataType.ItemIndex of
      0: Reg.WriteString(N, Value);
      1: Reg.WriteFloat(N, StrToFloat(Value));
      2: Reg.WriteInteger(N, StrToInt(Value));
      3: Reg.WriteCurrency(N, StrToCurr(Value));
      4: Reg.WriteBool(N, StrToBool(Value));
      5: Reg.WriteDate(N, StrToDate(Value));
      6: Reg.WriteDateTime(N, StrToDateTime(Value));
      7: Reg.WriteTime(N, StrToTime(Value));
      8: begin
        S := Value;
        Reg.WriteBinaryData(N, S[1], Length(S));
      end;
    else
      raise Exception.CreateFmt(SInvalidItemIndex, [cboDataType.ItemIndex]);
    end;
  finally
    Reg.Free;
  end;
end;

procedure TForm1.PositionRegistry(Reg: TEncryptedRegistry; CanCreate: Boolean);
begin
  Reg.LazyWrite := True;
  Reg.RootKey := CurrentKey;
  if not Reg.OpenKey(txtPath.Text, CanCreate) then begin
    raise Exception.CreateFmt('Error opening registry for name %s.',
                              [QuotedStr(txtPath.Text)]);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  cboKey.ItemIndex := 0;
  cboDataType.ItemIndex := 0;
end;

procedure TForm1.cmdGetValueClick(Sender: TObject);
begin
  ShowMessage(CurrentValue);
end;

procedure TForm1.cmdSetValueClick(Sender: TObject);
var
  Value: string;
begin
  if InputQuery('Value', 'Enter new value:', Value) then begin
    SetValue(Value);
  end;
end;

procedure TForm1.cmdCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.cmdQueryDataTypeClick(Sender: TObject);
const
  RegDataTypeToStr: array [TRegDataType] of string = (
    'Unknown', 'String', 'ExpandString', 'Integer', 'Binary'
  );
var
  Reg: TEncryptedRegistry;
begin
  Reg := TEncryptedRegistry.Create(TBlowfishCipher, EncryptionKey,
                                   SizeOf(EncryptionKey));
  try
    PositionRegistry(Reg, False);
    ShowMessage(RegDataTypeToStr[Reg.GetDataType(txtName.Text)]);
  finally
    Reg.Free;
  end;
end;

end.
